-- =======================================================================================
--  C O A L I T I O N   R U L E S
-- =======================================================================================

-- This method check, for the selected unit types, if a unit of the selected coalition
-- is in the selected zone
-- @param coalition           selected coalition
-- @param plane               flag telling if the plane type is to be processed
-- @param helicopter          flag telling if the helicopter type is to be processed
-- @param ship                flag telling if the ship type is to be processed
-- @param vehicle             flag telling if the vehicle type is to be processed
-- @param zoneId              selected zone
-- @return true/false         true if a unit is found in the zone
function c_coalition_in_zone( coalition, plane, helicopter, ship, vehicle, zoneId)
  -- Loop over countries of this coalition
  for i, countryInfo in ipairs( mission.coalition[coalition].country) do
  
    -- Process planes if requested
    if plane ~= "" then
      for j, groupInfo in ipairs( countryInfo["plane"].group) do
        for k, unitInfo in ipairs( groupInfo.units) do
          if c_unit_in_zone( unitInfo.unitId, zoneId) then
            -- At least one plane is in the zone
            return true
          end
        end
      end
    end
    
    -- Process helicopter if requested
    if helicopter ~= "" then
      for j, groupInfo in ipairs( countryInfo["helicopter"].group) do
        for k, unitInfo in ipairs( groupInfo.units) do
          if c_unit_in_zone( unitInfo.unitId, zoneId) then
            -- At least one helicopter is in the zone
            return true
          end
        end
      end
    end
    
    -- Process ship if requested
    if ship ~= "" then
      for j, groupInfo in ipairs( countryInfo["ship"].group) do
        for k, unitInfo in ipairs( groupInfo.units) do
          if c_unit_in_zone( unitInfo.unitId, zoneId) then
            -- At least one ship is in the zone
            return true
          end
        end
      end
    end
    
    -- Process vehicle if requested
    if vehicle ~= "" then
      for j, groupInfo in ipairs( countryInfo["vehicle"].group) do
        for k, unitInfo in ipairs( groupInfo.units) do
          if c_unit_in_zone( unitInfo.unitId, zoneId) then
            -- At least one vehicle is in the zone
            return true
          end
        end
      end
    end
  end
  
  -- No unit, among selected types, is in the zone
  return false
end


-- This method check, for the selected unit types, if all the units of the selected coalition
-- are out of the selected zone
-- @param coalition           selected coalition
-- @param plane               flag telling if the plane type is to be processed
-- @param helicopter          flag telling if the helicopter type is to be processed
-- @param ship                flag telling if the ship type is to be processed
-- @param vehicle             flag telling if the vehicle type is to be processed
-- @param zoneId              selected zone
-- @return true/false         true if all units are found out of the zone
function c_coalition_out_zone( coalition, plane, helicopter, ship, vehicle, zoneId)
  -- Loop over countries of this coalition
  for i, countryInfo in ipairs( mission.coalition[coalition].country) do
  
    -- Process planes if requested
    if plane ~= "" then
      for j, groupInfo in ipairs( countryInfo["plane"].group) do
        for k, unitInfo in ipairs( groupInfo.units) do
          if c_unit_in_zone( unitInfo.unitId, zoneId) then
            -- At least one plane is in the zone
            return false
          end
        end
      end
    end
    
    -- Process helicopter if requested
    if helicopter ~= "" then
      for j, groupInfo in ipairs( countryInfo["helicopter"].group) do
        for k, unitInfo in ipairs( groupInfo.units) do
          if c_unit_in_zone( unitInfo.unitId, zoneId) then
            -- At least one helicopter is in the zone
            return false
          end
        end
      end
    end
    
    -- Process ship if requested
    if ship ~= "" then
      for j, groupInfo in ipairs( countryInfo["ship"].group) do
        for k, unitInfo in ipairs( groupInfo.units) do
          if c_unit_in_zone( unitInfo.unitId, zoneId) then
            -- At least one ship is in the zone
            return false
          end
        end
      end
    end
    
    -- Process vehicle if requested
    if vehicle ~= "" then
      for j, groupInfo in ipairs( countryInfo["vehicle"].group) do
        for k, unitInfo in ipairs( groupInfo.units) do
          if c_unit_in_zone( unitInfo.unitId, zoneId) then
            -- At least one vehicle is in the zone
            return false
          end
        end
      end
    end
  end
  
  -- No unit, among selected types, is in the zone
  return true
end


-- This method check, for the selected unit types, if a unit of the selected coalition
-- is in the selected moving zone
-- @param coalition           selected coalition
-- @param plane               flag telling if the plane type is to be processed
-- @param helicopter          flag telling if the helicopter type is to be processed
-- @param ship                flag telling if the ship type is to be processed
-- @param vehicle             flag telling if the vehicle type is to be processed
-- @param zoneId              selected zone
-- @param unitId              selected unit used to move the zone the zone
-- @return true/false         true if a unit is found in the zone
function c_coalition_in_zone_unit( coalition, plane, helicopter, ship, vehicle, zoneId, unitId)
  -- Loop over countries of this coalition
  for i, countryInfo in ipairs( mission.coalition[coalition].country) do
  
    -- Process planes if requested
    if plane ~= "" then
      for j, groupInfo in ipairs( countryInfo["plane"].group) do
        for k, unitInfo in ipairs( groupInfo.units) do
          if c_unit_in_zone_unit( unitInfo.unitId, zoneId, unitId) then
            -- At least one plane is in the zone
            return true
          end
        end
      end
    end
    
    -- Process helicopter if requested
    if helicopter ~= "" then
      for j, groupInfo in ipairs( countryInfo["helicopter"].group) do
        for k, unitInfo in ipairs( groupInfo.units) do
          if c_unit_in_zone_unit( unitInfo.unitId, zoneId, unitId) then
            -- At least one helicopter is in the zone
            return true
          end
        end
      end
    end
    
    -- Process ship if requested
    if ship ~= "" then
      for j, groupInfo in ipairs( countryInfo["ship"].group) do
        for k, unitInfo in ipairs( groupInfo.units) do
          if c_unit_in_zone_unit( unitInfo.unitId, zoneId, unitId) then
            -- At least one ship is in the zone
            return true
          end
        end
      end
    end
    
    -- Process vehicle if requested
    if vehicle~= "" then
      for j, groupInfo in ipairs( countryInfo["vehicle"].group) do
        for k, unitInfo in ipairs( groupInfo.units) do
          if c_unit_in_zone_unit( unitInfo.unitId, zoneId, unitId) then
            -- At least one vehicle is in the zone
            return true
          end
        end
      end
    end
  end
  
  -- No unit, among selected types, is in the zone
  return false
end


-- This method check, for the selected unit types, if all the units of the selected coalition
-- are out of the selected moving zone
-- @param coalition           selected coalition
-- @param plane               flag telling if the plane type is to be processed
-- @param helicopter          flag telling if the helicopter type is to be processed
-- @param ship                flag telling if the ship type is to be processed
-- @param vehicle             flag telling if the vehicle type is to be processed
-- @param zoneId              selected zone
-- @param unitId              selected unit used to move the zone the zone
-- @return true/false         true if all units are found out of the zone
function c_coalition_out_zone_unit( coalition, plane, helicopter, ship, vehicle, zoneId, unitId)
  -- Loop over countries of this coalition
  for i, countryInfo in ipairs( mission.coalition[coalition].country) do
  
    -- Process planes if requested
    if plane ~= "" then
      for j, groupInfo in ipairs( countryInfo["plane"].group) do
        for k, unitInfo in ipairs( groupInfo.units) do
          if c_unit_in_zone_unit( unitInfo.unitId, zoneId, unitId) then
            -- At least one plane is in the zone
            return false
          end
        end
      end
    end
    
    -- Process helicopter if requested
    if helicopter ~= "" then
      for j, groupInfo in ipairs( countryInfo["helicopter"].group) do
        for k, unitInfo in ipairs( groupInfo.units) do
          if c_unit_in_zone_unit( unitInfo.unitId, zoneId, unitId) then
            -- At least one helicopter is in the zone
            return false
          end
        end
      end
    end
    
    -- Process ship if requested
    if ship ~= "" then
      for j, groupInfo in ipairs( countryInfo["ship"].group) do
        for k, unitInfo in ipairs( groupInfo.units) do
          if c_unit_in_zone_unit( unitInfo.unitId, zoneId, unitId) then
            -- At least one ship is in the zone
            return false
          end
        end
      end
    end
    
    -- Process vehicle if requested
    if vehicle ~= "" then
      for j, groupInfo in ipairs( countryInfo["vehicle"].group) do
        for k, unitInfo in ipairs( groupInfo.units) do
          if c_unit_in_zone_unit( unitInfo.unitId, zoneId, unitId) then
            -- At least one vehicle is in the zone
            return false
          end
        end
      end
    end
  end
  
  -- No unit, among selected types, is in the zone
  return true
end



-- =======================================================================================
--  C A L L S I G N   R U L E S
-- =======================================================================================

-- Check if a unit with the specified callsign in in the specified zone
-- @param coalition           selected coalition, just to speed up the process
-- @param plane               flag telling if the plane type is to be processed
-- @param helicopter          flag telling if the helicopter type is to be processed
-- @param callsign            callsign to process
-- @param zoneId              selected zone
function c_callsign_in_zone( coalition, plane, helicopter, callsign, zoneId)
  if callsign then
    for i, countryInfo in ipairs( mission.coalition[coalition].country) do
      if plane ~= "" then
        for j, groupInfo in ipairs( countryInfo["plane"].group) do
          for k, unitInfo in ipairs( groupInfo.units) do
            if unitInfo.callsign.name == callsign then
              if c_unit_in_zone( unitInfo.unitId, zoneId) then
                -- At least one plane with this callsign is in the zone
                return true
              end
            end
          end
        end
      end

      if helicopter ~= "" then
        for j, groupInfo in ipairs( countryInfo["helicopter"].group) do
          for k, unitInfo in ipairs( groupInfo.units) do
            if unitInfo.callsign.name == callsign then
              if c_unit_in_zone( unitInfo.unitId, zoneId) then
                -- At least one plane with this callsign is in the zone
                return true
              end
            end
          end
        end
      end
      
    end
  end
  return false
end

-- Check if no unit with the specified callsign is in the specified zone
-- @param coalition           selected coalition, just to speed up the process
-- @param plane               flag telling if the plane type is to be processed
-- @param helicopter          flag telling if the helicopter type is to be processed
-- @param callsign            callsign to process
-- @param zoneId              selected zone
function c_callsign_out_zone( coalition, plane, helicopter, callsign, zoneId)
  if callsign then
    for i, countryInfo in ipairs( mission.coalition[coalition].country) do
      if plane ~= "" then
        for j, groupInfo in ipairs( countryInfo["plane"].group) do
          for k, unitInfo in ipairs( groupInfo.units) do
            if unitInfo.callsign.name == callsign then
              if c_unit_in_zone( unitInfo.unitId, zoneId) then
                -- At least one plane with this callsign is in the zone
                return false
              end
            end
          end
        end
      end

      if helicopter ~= "" then
        for j, groupInfo in ipairs( countryInfo["helicopter"].group) do
          for k, unitInfo in ipairs( groupInfo.units) do
            if unitInfo.callsign.name == callsign then
              if c_unit_in_zone( unitInfo.unitId, zoneId) then
                -- At least one plane with this callsign is in the zone
                return false
              end
            end
          end
        end
      end
      
    end
  end
  return true
end

-- Check if a unit with the specified callsign in in the specified zone
-- @param coalition           selected coalition, just to speed up the process
-- @param plane               flag telling if the plane type is to be processed
-- @param helicopter          flag telling if the helicopter type is to be processed
-- @param callsign            callsign to process
-- @param zoneId              selected zone
-- @param unitId              selected unit used to move the zone the zone
function c_callsign_in_zone_unit( coalition, plane, helicopter, callsign, zoneId, unitId)
  if callsign then
    for i, countryInfo in ipairs( mission.coalition[coalition].country) do
      if plane ~= "" then
        for j, groupInfo in ipairs( countryInfo["plane"].group) do
          for k, unitInfo in ipairs( groupInfo.units) do
            if unitInfo.callsign.name == callsign then
              if c_unit_in_zone_unit( unitInfo.unitId, zoneId, unitId) then
                -- At least one plane with this callsign is in the zone
                return true
              end
            end
          end
        end
      end

      if helicopter ~= "" then
        for j, groupInfo in ipairs( countryInfo["helicopter"].group) do
          for k, unitInfo in ipairs( groupInfo.units) do
            if unitInfo.callsign.name == callsign then
              if c_unit_in_zone_unit( unitInfo.unitId, zoneId, unitId) then
                -- At least one plane with this callsign is in the zone
                return true
              end
            end
          end
        end
      end
      
    end
  end
  return false
end

-- Check if no unit with the specified callsign is in the specified zone
-- @param coalition           selected coalition, just to speed up the process
-- @param plane               flag telling if the plane type is to be processed
-- @param helicopter          flag telling if the helicopter type is to be processed
-- @param callsign            callsign to process
-- @param zoneId              selected zone
-- @param unitId              selected unit used to move the zone the zone
function c_callsign_out_zone_unit( coalition, plane, helicopter, callsign, zoneId, unitId)
  if callsign then
    for i, countryInfo in ipairs( mission.coalition[coalition].country) do
      if plane ~= "" then
        for j, groupInfo in ipairs( countryInfo["plane"].group) do
          for k, unitInfo in ipairs( groupInfo.units) do
            if unitInfo.callsign.name == callsign then
              if c_unit_in_zone_unit( unitInfo.unitId, zoneId, unitId) then
                -- At least one plane with this callsign is in the zone
                return false
              end
            end
          end
        end
      end

      if helicopter ~= "" then
        for j, groupInfo in ipairs( countryInfo["helicopter"].group) do
          for k, unitInfo in ipairs( groupInfo.units) do
            if unitInfo.callsign.name == callsign then
              if c_unit_in_zone_unit( unitInfo.unitId, zoneId, unitId) then
                -- At least one plane with this callsign is in the zone
                return false
              end
            end
          end
        end
      end
      
    end
  end
  return true
end



-- =======================================================================================
--  G R O U P   R U L E S
-- =======================================================================================

function c_group_number_lower_than( coalition, groupId, threshold)
  local unitNumber = 0
  for i, countryInfo in ipairs( mission.coalition[coalition].country) do
    for j, groupInfo in ipairs( countryInfo["plane"].group) do
      if groupInfo.groupId == tonumber(groupId) then
        for k, unitInfo in ipairs( groupInfo.units) do
          if c_unit_alive( unitInfo.unitId) then
            unitNumber = unitNumber + 1
          end
        end
        if unitNumber >= tonumber( threshold) then
          return false
        else
          return true
        end
      end
    end
  end
    
  for i, countryInfo in ipairs( mission.coalition[coalition].country) do
    for j, groupInfo in ipairs( countryInfo["helicopter"].group) do
      if groupInfo.groupId == tonumber(groupId) then
        for k, unitInfo in ipairs( groupInfo.units) do
          if c_unit_alive( unitInfo.unitId) then
            unitNumber = unitNumber + 1
          end
        end
        if unitNumber >= tonumber( threshold) then
          return false
        else
          return true
        end
      end
    end
  end
  
  for i, countryInfo in ipairs( mission.coalition[coalition].country) do  
    for j, groupInfo in ipairs( countryInfo["vehicle"].group) do
      if groupInfo.groupId == tonumber(groupId) then
        for k, unitInfo in ipairs( groupInfo.units) do
          if c_unit_alive( unitInfo.unitId) then
            unitNumber = unitNumber + 1
          end
        end
        if unitNumber >= tonumber( threshold) then
          return false
        else
          return true
        end
      end
    end
  end
    
  for i, countryInfo in ipairs( mission.coalition[coalition].country) do  
    for j, groupInfo in ipairs( countryInfo["ship"].group) do
      if groupInfo.groupId == tonumber(groupId) then
        for k, unitInfo in ipairs( groupInfo.units) do
          if c_unit_alive( unitInfo.unitId) then
            unitNumber = unitNumber + 1
          end
        end
        if unitNumber >= tonumber( threshold) then
          return false
        else
          return true
        end
      end
    end     
  end
  return false
end


-- =======================================================================================
--  F I L E   R U L E S
-- =======================================================================================

-- Method used to check if a file exits or not
-- @param fullFilePath        full path of the file (file must be empty)
function c_file_exits( fullFilePath)
  fId = _G.loadfile( fullFilePath)
  if fId then
    -- File exits
    return true
  else
    -- File does not exit
    return false
  end
end



-- =======================================================================================
--  M I S C  R U L E S
-- =======================================================================================

-- Method used for debug coding purpose
function getLoInterface()
  -- Interface
  base = getfenv(0)
  text = ""
  for key, value in pairs( base) do
    if key == "require" then
      text = text .. "NewKey: " .. key .. "\n"
    end
  end
  a_out_text_delay(text, 10)
  return false
end
